import 'package:flutter/material.dart';

mixin OneValueFieldMixin {
  late final TextEditingController _controller;

  /// Listener added to the controller.
  void controllerListener({final Function(dynamic value)? onChanged}) {
    onChanged?.call(value);
  }

  /// Call this on disposing.
  void disposeMixin() {
    _controller.removeListener(controllerListener);
    _controller.dispose();
  }

  /// Call this on initializing.
  void initStateMixin(
      {final TextEditingController? controller,
      final dynamic initialValue,
      final Function(dynamic value)? onChanged}) {
    if (controller == null) {
      _controller = TextEditingController();
    } else {
      _controller = controller;
    }
    _controller.addListener(() => controllerListener(onChanged: onChanged));
    if (initialValue != null) {
      _controller.text = format(initialValue);
      _controller.selection =
          TextSelection.collapsed(offset: _controller.text.length);
    }
  }

  TextEditingController get controller => _controller;

  String format(final dynamic value) => value;

  dynamic get value => _controller.text;
}
